#
# Name:
#   device_data.rb
#
# Created by: GaddyGaddy
#
# Description:
#   
# Will handle data about the client like node info from Ohai that will tell everything like ip-address, macaddress,
#  hw config etc. The data will be sent to the GaddyGaddy service
#
# Copyright (c) 2013 GaddyGaddy
#
# All rights reserved.
# 

require_relative '../logging/logging'
require 'ohai'

class DeviceInfo
  include Logging
  extend Logging

  def initialize(host, gg_config)
    @host = host
    @gg_config = gg_config
  end

  # Give the host name set previous
  def host
    @host
  end

  def gg_config
    @gg_config
  end

  # Get data about the client by using the Ohai component developed by Opscode
  def get_ohai_data
    Ohai::Config[:plugin_path] << '/etc/chef/ohai_plugins'
    ohai_system = Ohai::System.new
    ohai_system.all_plugins
    ohai_system.data
  end

  # Get the device data, if the data does not exist get it from Ohai
  def data
    @data||=get_ohai_data
  end

  # Will post the device data to the client service
  def post
    url = Request.get_base_url(host) + "/device/set_device_data/1/#{gg_config.config[:user_id_salt]}/#{gg_config.config[:device_id]}/#{gg_config.config[:token]}"
    logger.debug "Will post this ohai data:\n #{data}"
    response = Request.client_service_post url, :ohai_data => data
    logger.debug "The response for the request is #{response} #{response.class}"
    raise JCouldNotPostClientDataException.new({:message=> "Could not post data to the gaddygaddy service, error code is #{response.body}"}) unless response[:status].to_i == 0
  end

end