#
# Name:
#   logging.rb
#
# Created by: GaddyGaddy
#
# Description:
#   
#
#
# Copyright (c) 2013 GaddyGaddy
#
# All rights reserved.
# 

require 'log4r'
require 'log4r/outputter/datefileoutputter'
require 'log4r/yamlconfigurator'

module Logging
  # This is the magical bit that gets mixed into your classes
  @@log_conf = const_defined?(:APPLICATION_NAME) ? APPLICATION_NAME : "default"

  def logger
    Logging.logger
  end

  def set_log_level(level_str)
    logger.level = (Log4r::Log4rConfig::LogLevels.index level_str.upcase) + 1
  end

  def set_log_file(new_filename)
    format = Log4r::PatternFormatter.new(:pattern => "[%5l] %d :: %m", :date_pattern => '%y%m%d %H:%M:%S')
    dir_name = File.dirname(new_filename)
    file_name = File.basename(new_filename)
    logger.outputters << Log4r::DateFileOutputter.new('log_file', :filename => file_name, :dirname => dir_name, :formatter => format, :date_pattern => '%Y%m%d')
  end

  def self.set_log_conf log_conf
    @@log_conf = log_conf
  end

  # create a new log4r logger with config

  def self.create_logger
    log_cfg = Log4r::YamlConfigurator
    conf_file = File.join(File.dirname(__FILE__), "..", "..", "conf","log4r.yml")
    log_cfg.load_yaml_file(conf_file)

    # app logger
    new_logger = Log4r::Logger[@@log_conf]
    raise "Missing logger config #{@@log_conf} in log config file #{conf_file}" unless new_logger
    new_logger.level = Log4r::INFO
    new_logger.info "Log initiated"
    new_logger
  end

  # Global, memoized, lazy initialized instance of a logger
  def self.logger
    @logger ||= create_logger
  end
end